==> pyform-0.0.4.tar.gz: pyform-0.0.4/tests/unit/test_returnseries.py <==
import datetime
import pytest
import pandas as pd
from pyform.returnseries import ReturnSeries, CashSeries

returns = ReturnSeries.read_csv("tests/unit/data/twitter_returns.csv")
spy = ReturnSeries.read_csv("tests/unit/data/spy_returns.csv")
qqq = ReturnSeries.read_csv("tests/unit/data/qqq_returns.csv")
libor1m = ReturnSeries.read_csv("tests/unit/data/libor1m_returns.csv")


def test_init():

    df = pd.read_csv("tests/unit/data/twitter_returns.csv")
    ReturnSeries(df, "Twitter")


def test_to_period():

    assert returns.to_week().iloc[1, 0] == 0.055942142480424284
    assert returns.to_month().iloc[1, 0] == 0.5311520760874386
    assert returns.to_quarter().iloc[1, 0] == -0.2667730077753935
    assert returns.to_year().iloc[1, 0] == -0.4364528678695403
    assert returns.to_week("arithmetic").iloc[1, 0] == 0.05658200000000001
    assert returns.to_week("continuous").iloc[1, 0] == 0.05821338474015869
    with pytest.raises(ValueError):
        returns.to_period("W", "contnuuous")  # typo in continuous should cause failure
    with pytest.raises(ValueError):
        returns.to_period("H", "geometric")  # converting data to higher frequency


def test_add_benchmark():

    returns.add_benchmark(spy, "S&P 500")
    assert "S&P 500" in returns.benchmark

    returns.add_benchmark(qqq)
    assert "QQQ" in returns.benchmark


def test_add_risk_free():

    returns.add_risk_free(libor1m, "libor")
    assert "libor" in returns.risk_free

    returns.add_risk_free(libor1m)
    assert "LIBOR_1M" in returns.risk_free


def test_corr():

    returns = ReturnSeries.read_csv("tests/unit/data/twitter_returns.csv")

    # no benchmark should raise ValueError
    with pytest.raises(ValueError):
        returns.get_corr()

    # with single benchmark
    returns.add_benchmark(spy)

    corr = returns.get_corr()
    expected_output = pd.DataFrame(
        data={"name": ["SPY"], "field": "correlation", "value": [0.21224719919904408]}
    )
    assert corr.equals(expected_output)

    corr = returns.get_corr(meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["SPY"],
            "field": "correlation",
            "value": [0.21224719919904408],
            "freq": "M",
            "method": "pearson",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
            "total": 80,
            "skipped": 0,
        }
    )
    assert corr.equals(expected_output)

    # test multiple benchmarks
    returns.add_benchmark(qqq)
    corr = returns.get_corr()
    expected_output = pd.DataFrame(
        data={
            "name": ["SPY", "QQQ"],
            "field": "correlation",
            "value": [0.21224719919904408, 0.27249109347246325],
        }
    )
    assert corr.equals(expected_output)


def test_total_return():

    returns = ReturnSeries.read_csv("tests/unit/data/twitter_returns.csv")

    # No benchmark
    total_return = returns.get_total_return()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "total return",
            "value": [-0.35300922502128473],
        }
    )
    assert total_return.equals(expected_output)

    # with single benchmark
    returns.add_benchmark(spy)
    total_return = returns.get_total_return()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY"],
            "field": "total return",
            "value": [-0.35300922502128473, 0.6935467657365115],
        }
    )
    assert total_return.equals(expected_output)

    # meta=True
    total_return = returns.get_total_return(meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY"],
            "field": "total return",
            "value": [-0.35300922502128473, 0.6935467657365115],
            "method": "geometric",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert total_return.equals(expected_output)

    # has benchmark, but include_bm=False
    total_return = returns.get_total_return(include_bm=False)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "total return",
            "value": [-0.35300922502128473],
        }
    )
    assert total_return.equals(expected_output)

    # test multiple benchmarks
    returns.add_benchmark(qqq)
    total_return = returns.get_total_return()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY", "QQQ"],
            "field": "total return",
            "value": [-0.35300922502128473, 0.6935467657365115, 1.894217403555647],
        }
    )
    assert total_return.equals(expected_output)


def test_annualized_return():

    returns = ReturnSeries.read_csv("tests/unit/data/twitter_returns.csv")

    # No benchmark
    ann_return = returns.get_annualized_return()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized return",
            "value": [-0.06352921539090761],
        }
    )
    assert ann_return.equals(expected_output)

    ann_return = returns.get_annualized_return(method="arithmetic", meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized return",
            "value": [0.08553588206768473],
            "method": "arithmetic",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert ann_return.equals(expected_output)

    ann_return = returns.get_annualized_return(method="continuous")
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized return",
            "value": [0.08553588206768473],
        }
    )
    assert ann_return.equals(expected_output)

    # with single benchmark
    returns.add_benchmark(spy)
    ann_return = returns.get_annualized_return()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY"],
            "field": "annualized return",
            "value": [-0.06352921539090761, 0.08265365923419554],
        }
    )
    assert ann_return.equals(expected_output)

    # meta=True
    ann_return = returns.get_annualized_return(meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY"],
            "field": "annualized return",
            "value": [-0.06352921539090761, 0.08265365923419554],
            "method": "geometric",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert ann_return.equals(expected_output)

    # has benchmark, but include_bm=False
    ann_return = returns.get_annualized_return(include_bm=False)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized return",
            "value": [-0.06352921539090761],
        }
    )
    assert ann_return.equals(expected_output)


def test_annualized_volatility():

    returns = ReturnSeries.read_csv("tests/unit/data/twitter_returns.csv")

    # No benchmark
    ann_vol = returns.get_annualized_volatility()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized volatility",
            "value": [0.5200932110481337],
        }
    )
    assert ann_vol.equals(expected_output)

    # daily volatility
    ann_vol = returns.get_annualized_volatility(freq="D", meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized volatility",
            "value": [0.545298443138832],
            "freq": "D",
            "method": "sample",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert ann_vol.equals(expected_output)

    # population standard deviation
    ann_vol = returns.get_annualized_volatility(method="population", meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized volatility",
            "value": [0.5168324064202331],
            "freq": "M",
            "method": "population",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert ann_vol.equals(expected_output)

    # with single benchmark
    returns.add_benchmark(spy)
    ann_vol = returns.get_annualized_volatility()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY"],
            "field": "annualized volatility",
            "value": [0.5200932110481337, 0.13609234804383752],
        }
    )
    assert ann_vol.equals(expected_output)

    # daily volatility
    ann_vol = returns.get_annualized_volatility(freq="D", meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY"],
            "field": "annualized volatility",
            "value": [0.545298443138832, 0.17501475527479404],
            "freq": "D",
            "method": "sample",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert ann_vol.equals(expected_output)

    # has benchmark, but include_bm=False
    ann_vol = returns.get_annualized_volatility(include_bm=False)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized volatility",
            "value": [0.5200932110481337],
        }
    )
    assert ann_vol.equals(expected_output)


def test_sharpe_ratio():

    returns = ReturnSeries.read_csv("tests/unit/data/spy_returns.csv")

    # No benchmark
    sharpe_ratio = returns.get_sharpe_ratio()
    expected_output = pd.DataFrame(
        data={"name": ["SPY"], "field": "sharpe ratio", "value": [0.5319582050650019]}
    )
    assert sharpe_ratio.equals(expected_output)

    # daily
    sharpe_ratio = returns.get_sharpe_ratio(freq="D", meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["SPY"],
            "field": "sharpe ratio",
            "value": [0.392952489244965],
            "freq": "D",
            "risk_free": "cash_0: 0.0%",
            "start": datetime.datetime.strptime("2003-04-01", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert sharpe_ratio.equals(expected_output)

    # use libor for risk free rate
    returns.add_risk_free(libor1m, "libor")
    sharpe_ratio = returns.get_sharpe_ratio(freq="D", risk_free="libor", meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["SPY"],
            "field": "sharpe ratio",
            "value": [0.31751335834676475],
            "freq": "D",
            "risk_free": "LIBOR_1M: 1.54%",
            "start": datetime.datetime.strptime("2003-04-01", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-19", "%Y-%m-%d"),
        }
    )
    assert sharpe_ratio.equals(expected_output)

    # with benchmark
    returns.add_benchmark(qqq)
    sharpe_ratio = returns.get_sharpe_ratio(meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["SPY", "QQQ"],
            "field": "sharpe ratio",
            "value": [0.5319582050650019, 0.8028838684875361],
            "freq": "M",
            "risk_free": "cash_0: 0.0%",
            "start": datetime.datetime.strptime("2003-04-01", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert sharpe_ratio.equals(expected_output)

    # wrong key
    with pytest.raises(ValueError):
        returns.get_sharpe_ratio(risk_free="not-exist")

    # wrong type
    with pytest.raises(TypeError):
        returns.get_sharpe_ratio(risk_free=libor1m)


def test_libor_fred():

    CashSeries.read_fred_libor_1m()

==> pyform-0.0.5.tar.gz: pyform-0.0.5/tests/unit/test_returnseries.py <==
import datetime
import pytest
import pandas as pd
from pyform.returnseries import ReturnSeries, CashSeries

returns = ReturnSeries.read_csv("tests/unit/data/twitter_returns.csv")
spy = ReturnSeries.read_csv("tests/unit/data/spy_returns.csv")
qqq = ReturnSeries.read_csv("tests/unit/data/qqq_returns.csv")
libor1m = ReturnSeries.read_csv("tests/unit/data/libor1m_returns.csv")


def test_init():

    df = pd.read_csv("tests/unit/data/twitter_returns.csv")
    ReturnSeries(df, "Twitter")


def test_to_period():

    assert returns.to_week().iloc[1, 0] == 0.055942142480424284
    assert returns.to_month().iloc[1, 0] == 0.5311520760874386
    assert returns.to_quarter().iloc[1, 0] == -0.2667730077753935
    assert returns.to_year().iloc[1, 0] == -0.4364528678695403

    with pytest.raises(ValueError):
        returns.to_period("H", "geometric")  # converting data to higher frequency


def test_add_bm():

    returns.add_bm(spy, "S&P 500")
    assert "S&P 500" in returns.benchmark

    returns.add_bm(qqq)
    assert "QQQ" in returns.benchmark


def test_add_rf():

    returns.add_rf(libor1m, "libor")
    assert "libor" in returns.risk_free

    returns.add_rf(libor1m)
    assert "LIBOR_1M" in returns.risk_free


def test_corr():

    returns = ReturnSeries.read_csv("tests/unit/data/twitter_returns.csv")

    # no benchmark should raise ValueError
    with pytest.raises(ValueError):
        returns.get_corr()

    # with single benchmark
    returns.add_bm(spy)

    corr = returns.get_corr()
    expected_output = pd.DataFrame(
        data={"name": ["SPY"], "field": "correlation", "value": [0.21224719919904408]}
    )
    assert corr.equals(expected_output)

    corr = returns.get_corr(meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["SPY"],
            "field": "correlation",
            "value": [0.21224719919904408],
            "freq": "M",
            "method": "pearson",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
            "total": 80,
            "used": 80,
        }
    )
    assert corr.equals(expected_output)

    # test multiple benchmarks
    returns.add_bm(qqq)
    corr = returns.get_corr()
    expected_output = pd.DataFrame(
        data={
            "name": ["SPY", "QQQ"],
            "field": "correlation",
            "value": [0.21224719919904408, 0.27249109347246325],
        }
    )
    assert corr.equals(expected_output)


def test_total_return():

    returns = ReturnSeries.read_csv("tests/unit/data/twitter_returns.csv")

    # No benchmark
    total_return = returns.get_tot_ret()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "total return",
            "value": [-0.35300922502128473],
        }
    )
    assert total_return.equals(expected_output)

    # with single benchmark
    returns.add_bm(spy)
    total_return = returns.get_tot_ret()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY"],
            "field": "total return",
            "value": [-0.35300922502128473, 0.6935467657365115],
        }
    )
    assert total_return.equals(expected_output)

    # meta=True
    total_return = returns.get_tot_ret(meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY"],
            "field": "total return",
            "value": [-0.35300922502128473, 0.6935467657365115],
            "method": "geometric",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert total_return.equals(expected_output)

    # has benchmark, but include_bm=False
    total_return = returns.get_tot_ret(include_bm=False)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "total return",
            "value": [-0.35300922502128473],
        }
    )
    assert total_return.equals(expected_output)

    # test multiple benchmarks
    returns.add_bm(qqq)
    total_return = returns.get_tot_ret()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY", "QQQ"],
            "field": "total return",
            "value": [-0.35300922502128473, 0.6935467657365115, 1.894217403555647],
        }
    )
    assert total_return.equals(expected_output)


def test_annualized_return():

    returns = ReturnSeries.read_csv("tests/unit/data/twitter_returns.csv")

    # No benchmark
    ann_return = returns.get_ann_ret()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized return",
            "value": [-0.06352921539090761],
        }
    )
    assert ann_return.equals(expected_output)

    ann_return = returns.get_ann_ret(method="arithmetic", meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized return",
            "value": [0.08553588206768473],
            "method": "arithmetic",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert ann_return.equals(expected_output)

    ann_return = returns.get_ann_ret(method="continuous")
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized return",
            "value": [0.08553588206768473],
        }
    )
    assert ann_return.equals(expected_output)

    # with single benchmark
    returns.add_bm(spy)
    ann_return = returns.get_ann_ret()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY"],
            "field": "annualized return",
            "value": [-0.06352921539090761, 0.08265365923419554],
        }
    )
    assert ann_return.equals(expected_output)

    # meta=True
    ann_return = returns.get_ann_ret(meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY"],
            "field": "annualized return",
            "value": [-0.06352921539090761, 0.08265365923419554],
            "method": "geometric",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert ann_return.equals(expected_output)

    # has benchmark, but include_bm=False
    ann_return = returns.get_ann_ret(include_bm=False)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized return",
            "value": [-0.06352921539090761],
        }
    )
    assert ann_return.equals(expected_output)


def test_annualized_volatility():

    returns = ReturnSeries.read_csv("tests/unit/data/twitter_returns.csv")

    # No benchmark
    ann_vol = returns.calc_ann_vol()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized volatility",
            "value": [0.5200932110481337],
        }
    )
    assert ann_vol.equals(expected_output)

    # daily volatility
    ann_vol = returns.calc_ann_vol(freq="D", meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized volatility",
            "value": [0.5453033363906205],
            "freq": "D",
            "method": "sample",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert ann_vol.equals(expected_output)

    # population standard deviation
    ann_vol = returns.calc_ann_vol(method="population", meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized volatility",
            "value": [0.5168324064202331],
            "freq": "M",
            "method": "population",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert ann_vol.equals(expected_output)

    # with single benchmark
    returns.add_bm(spy)
    ann_vol = returns.calc_ann_vol()
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY"],
            "field": "annualized volatility",
            "value": [0.5200932110481337, 0.13609234804383752],
        }
    )
    assert ann_vol.equals(expected_output)

    # daily volatility
    ann_vol = returns.calc_ann_vol(freq="D", meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR", "SPY"],
            "field": "annualized volatility",
            "value": [0.5453033363906205, 0.17501297679149252],
            "freq": "D",
            "method": "sample",
            "start": datetime.datetime.strptime("2013-11-07", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert ann_vol.equals(expected_output)

    # has benchmark, but include_bm=False
    ann_vol = returns.calc_ann_vol(include_bm=False)
    expected_output = pd.DataFrame(
        data={
            "name": ["TWTR"],
            "field": "annualized volatility",
            "value": [0.5200932110481337],
        }
    )
    assert ann_vol.equals(expected_output)


def test_sharpe_ratio():

    returns = ReturnSeries.read_csv("tests/unit/data/spy_returns.csv")

    # No benchmark
    sharpe_ratio = returns.get_sharpe()
    expected_output = pd.DataFrame(
        data={"name": ["SPY"], "field": "sharpe ratio", "value": [0.5319582050650019]}
    )
    assert sharpe_ratio.equals(expected_output)

    # daily
    sharpe_ratio = returns.get_sharpe(freq="D", meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["SPY"],
            "field": "sharpe ratio",
            "value": [0.39295707447678757],
            "freq": "D",
            "risk_free": "cash_0: 0.0%",
            "start": datetime.datetime.strptime("2003-04-01", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert sharpe_ratio.equals(expected_output)

    # use libor for risk free rate
    returns.add_rf(libor1m, "libor")
    sharpe_ratio = returns.get_sharpe(freq="D", risk_free="libor", meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["SPY"],
            "field": "sharpe ratio",
            "value": [0.31755154920375406],
            "freq": "D",
            "risk_free": "LIBOR_1M: 1.54%",
            "start": datetime.datetime.strptime("2003-04-01", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-19", "%Y-%m-%d"),
        }
    )
    assert sharpe_ratio.equals(expected_output)

    # with benchmark
    returns.add_bm(qqq)
    sharpe_ratio = returns.get_sharpe(meta=True)
    expected_output = pd.DataFrame(
        data={
            "name": ["SPY", "QQQ"],
            "field": "sharpe ratio",
            "value": [0.5319582050650019, 0.8028838684875361],
            "freq": "M",
            "risk_free": "cash_0: 0.0%",
            "start": datetime.datetime.strptime("2003-04-01", "%Y-%m-%d"),
            "end": datetime.datetime.strptime("2020-06-26", "%Y-%m-%d"),
        }
    )
    assert sharpe_ratio.equals(expected_output)

    # wrong key
    with pytest.raises(ValueError):
        returns.get_sharpe(risk_free="not-exist")

    # wrong type
    with pytest.raises(TypeError):
        returns.get_sharpe(risk_free=libor1m)


def test_rolling_tot_ret():

    returns = ReturnSeries.read_csv("tests/unit/data/twitter_returns.csv")

    # No benchmark
    roll_tot_ret = returns.get_rolling_tot_ret()
    roll_twtr = roll_tot_ret["TWTR"]
    assert roll_twtr.index[0] == datetime.datetime.strptime("2016-10-31", "%Y-%m-%d")
    assert roll_twtr["TWTR"][0] == -0.6002237318946346

    # Daily, rolling 252 days
    roll_tot_ret = returns.get_rolling_tot_ret(window=252, freq="D")
    roll_twtr = roll_tot_ret["TWTR"]
    assert roll_twtr.index[0] == datetime.datetime.strptime("2014-11-06", "%Y-%m-%d")
    assert roll_twtr["TWTR"][0] == -0.09043080731969699

    returns.add_bm(spy)
    roll_tot_ret = returns.get_rolling_tot_ret()
    roll_twtr = roll_tot_ret["TWTR"]
    roll_spy = roll_tot_ret["SPY"]
    assert roll_twtr.index[0] == datetime.datetime.strptime("2016-10-31", "%Y-%m-%d")
    assert roll_twtr["TWTR"][0] == -0.6002237318946346
    assert roll_spy.index[0] == datetime.datetime.strptime("2016-10-31", "%Y-%m-%d")
    assert roll_spy["SPY"][0] == 0.1996927920869329


def test_rolling_ann_vol():

    returns = ReturnSeries.read_csv("tests/unit/data/twitter_returns.csv")

    # No benchmark
    roll_ann_vol = returns.get_rolling_ann_vol()
    roll_twtr = roll_ann_vol["TWTR"]
    assert roll_twtr.index[0] == datetime.datetime.strptime("2016-10-31", "%Y-%m-%d")
    assert roll_twtr["TWTR"][0] == 0.5791236929456373

    # Daily, rolling 252 days
    roll_ann_vol = returns.get_rolling_ann_vol(window=252, freq="D")
    roll_twtr = roll_ann_vol["TWTR"]
    assert roll_twtr.index[0] == datetime.datetime.strptime("2014-11-06", "%Y-%m-%d")
    assert roll_twtr["TWTR"][0] == 0.639203890663799

    returns.add_bm(spy)
    roll_ann_vol = returns.get_rolling_ann_vol()
    roll_twtr = roll_ann_vol["TWTR"]
    roll_spy = roll_ann_vol["SPY"]
    assert roll_twtr.index[0] == datetime.datetime.strptime("2016-10-31", "%Y-%m-%d")
    assert roll_twtr["TWTR"][0] == 0.5791236929456373
    assert roll_spy.index[0] == datetime.datetime.strptime("2016-10-31", "%Y-%m-%d")
    assert roll_spy["SPY"][0] == 0.10935207559750833


def test_libor_fred():

    CashSeries.read_fred_libor_1m()
